var proxyquire = require('proxyquire');
var httpMocks = require('node-mocks-http');
var chai = require('chai'),
  spies = require('chai-spies');
chai.use(spies);
var expect = chai.expect;
var logger = require('trucktrack-logging').logger;

var config = {
	"env": "dev",
	"tokenEndpoint": "http://localhost:8080/auth/check_token",
	"domainUrl": "localhost",
	"domainPort": "8080",
	"public": [
		"/",
		"/bol",
		"/invoice",
		"/invoice-test",
		"/stylesheets",
		"/favicon.ico"
	]
};

describe("tokenCheck", function() {
  it('should not save oauthKey if url is public', function (done) {
    var req = httpMocks.createRequest({
      method: 'GET',
      url: '/bol',
      headers: {
        'Content-Type': 'application/json',
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.130 Safari/537.36'
      }
    });
    var res = httpMocks.createResponse();

    var spy = chai.spy(function(req, res, app) {});
    function TokenUtil() {}
    TokenUtil.prototype.checkToken = function(token, callback) {
      spy();
      callback(null, {});
    };
    var tokenCheck = proxyquire('../lib/check', {
      'continuation-local-storage': {
        getNamespace: function(key) {
          return getNamespaceSpy(key);
        },
        createNamespace: function(key) {
          return createNamespaceSpy(key);
        }
      },
      './http': TokenUtil
    });

    tokenCheck(req, res, config, function() {
      expect(spy).not.to.have.been.called();
      done();
    });
  });

  it("should save oauthKey in continuation-local-storage", function(done) {
    var getNamespaceSpy = chai.spy(function(req, res, app) {});
    var setSpy = chai.spy(function(req, res, app) {});
    var createNamespaceSpy = chai.spy(function(req, res, app) {
      return {
        set: function(key, value) {
          setSpy(key, value);
        }
      }
    });

    function TokenUtil() {}
    TokenUtil.prototype.checkToken = function(token, callback) {
      logger.info('Check token');
      createNamespaceSpy('oauth');
      setSpy('key', 123);
      callback(null, {});
    };
    var tokenCheck = proxyquire('../lib/check', {
      'continuation-local-storage': {
        getNamespace: function(key) {
          return getNamespaceSpy(key);
        },
        createNamespace: function(key) {
          return createNamespaceSpy(key);
        }
      },
      './http': TokenUtil
    });
    var req = httpMocks.createRequest({
      method: 'GET',
      url: '/user/42',
      headers: {
        'Authorization': 'Barrier 123',
        'Content-Type': 'application/json',
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/43.0.2357.130 Safari/537.36'
      }
    });
    var res = httpMocks.createResponse();

    tokenCheck(req, res, config, function() {
      expect(createNamespaceSpy).to.have.been.called.with("oauth");
      expect(setSpy).to.have.been.called.with('key', 123);
      done();
    });
  });
});
